from pydantic import BaseModel
from typing import Any

from gando.config import SETTINGS

from rest_framework.response import Response
from rest_framework.views import APIView


class BaseAPI(APIView):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.__data = None

        self.__logs_message = []
        self.__infos_message = []
        self.__warnings_message = []
        self.__errors_message = []
        self.__exceptions_message = []

        self.__monitor: dict = dict()

        self.__status_code: int | None = None

        self.__headers: dict = dict()
        self.__cookies_for_set: list = list()
        self.__cookies_for_delete: list = list()

    def response_context(self, data=None):
        self.__data = data
        tmp = {
            'success': self.__success(),
            'status_code': self.get_status_code(),
            'has_warning': self.__has_warning(),
            'monitor': self.__monitor,
            'data': self.validate_data(),
            'many': self.__many(),
        }
        if self.__debug_status:
            tmp['messages'] = self.__messages()
            tmp['headers'] = self.get_headers()

        ret = tmp
        return ret

    def set_log_message(self, key, value):
        log = {key: value}
        self.__logs_message.append(log)

    def set_info_message(self, key, value):
        info = {key: value}
        self.__infos_message.append(info)

    def set_warning_message(self, key, value):
        warning = {key: value}
        self.__warnings_message.append(warning)

    def set_error_message(self, key, value):
        error = {key: value}
        self.__errors_message.append(error)

    def set_exception_message(self, key, value):
        exception = {key: value}
        self.__exceptions_message.append(exception)

    def set_headers(self, key, value):
        self.__headers[key] = value

    def get_headers(self):
        return self.__headers

    def __messages(self, ) -> dict:
        tmp = {
            'info': self.__infos_message,
            'warning': self.__warnings_message,
            'error': self.__errors_message,

        }
        if self.__debug_status:
            tmp['log'] = self.__logs_message
            tmp['exception'] = self.__exceptions_message

        ret = tmp
        return ret

    def __many(self):
        if isinstance(self.__data, list):
            return True
        return False

    def __success(self):
        if len(self.__errors_message) == 0 and len(self.__exceptions_message) == 0:
            return True
        return False

    def __has_warning(self):
        if len(self.__warnings_message) != 0:
            return True
        return False

    def set_status_code(self, value: int):
        self.__status_code = value

    def get_status_code(self):
        return self.__status_code or 200

    def set_monitor(self, key, value):
        if key in self.__allowed_monitor_keys:
            self.__monitor[key] = value

    @property
    def __allowed_monitor_keys(self):
        return SETTINGS.MONITOR_KEYS

    def validate_data(self):
        data = self.__data

        if data is None:
            tmp = {'result': {'message': self.__default_message()}}

        elif isinstance(data, str):
            tmp = {'result': {'message': data}}

        elif isinstance(data, list):
            tmp = {
                'count': len(data),
                'next': None,
                'previous': None,
                'results': data,
            }

        elif isinstance(data, dict):
            tmp = {'result': data}

        else:
            tmp = {'result': {}}

        ret = tmp
        return ret

    @property
    def __debug_status(self):
        return SETTINGS.DEBUG

    def response(self, output_data=None):
        self.helper()

        data = self.response_context(output_data)
        rsp = Response(
            data,
            status=self.get_status_code(),
            headers=self.get_headers(),
        )

        if self.__cookies_for_delete:
            for i in self.__cookies_for_delete:
                rsp.delete_cookie(i)

        if self.__cookies_for_set:
            for i in self.__cookies_for_set:
                rsp.set_cookie(**i)

        return rsp

    def get_host(self):
        return self.request._request._current_scheme_host

    def append_host_to_url(self, value):
        ret = f'{self.get_host()}{value}'
        return ret

    def get_media_url(self):
        from django.conf import settings

        ret = settings.MEDIA_URL
        return ret

    def convert_filename_to_url(self, file_name):
        ret = f'{self.get_media_url()}{file_name}'
        return ret

    def convert_filename_to_url_localhost(self, file_name):
        ret = f'{self.get_host()}{self.get_media_url()}{file_name}'
        return ret

    def helper(self):
        pass

    def __default_message(self):
        status_code = self.get_status_code()

        if 100 <= status_code < 200:
            msg = 'please wait...'

        elif 200 <= status_code < 300:
            msg = 'Your request has been successfully registered.'

        elif 300 <= status_code < 400:
            msg = 'The requirements for your request are not available.'

        elif 400 <= status_code < 500:
            if status_code == 400:
                msg = 'Bad Request...'

            elif status_code == 401:
                msg = {'result': {
                    'message': 'Your authentication information is not available.'}}

            elif status_code == 403:
                msg = 'You do not have access to this section.'

            elif status_code == 404:
                msg = 'There is no information about your request.'

            else:
                msg = 'There was an error in how to send the request.'

        elif 500 <= status_code:
            msg = 'The server is unable to respond to your request.'

        else:
            msg = 'Undefined.'

        return msg

    class Cookie(BaseModel):
        key: str
        value: Any = ""
        max_age: Any = None
        expires: Any = None
        path: Any = "/"
        domain: Any = None
        secure: Any = False
        httponly: Any = False
        samesite: Any = None

    def cookie_getter(self, key: str):
        ret = self.request.COOKIES.get(key)
        return ret

    def cookie_setter(self, key: str, **kwargs):
        ret = self.__cookies_for_set.append(self.Cookie(key=key, **kwargs).model_dump())
        return ret

    def cookie_deleter(self, key: str):
        ret = self.__cookies_for_delete.append(key)
        return ret



from abc import abstractmethod
from pydantic import BaseModel

from django.db.models import Model

from gando.architectures.services import BaseService

FIRST_KEYWORD = 'first'
LAST_KEYWORD = 'last'
MANY_KEYWORD = 'many'
OBJECT_TYPE_KEYWORD = 'is_object'


class BaseGetterService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.values = self.__values(*args)
        self.filters = self.__filters(**kwargs)

        self.many = self.__many(**kwargs)
        self.first_or_last = self.__first_or_last(*args, **kwargs)

        self.is_object = self.__is_object(**kwargs)

    @property
    @abstractmethod
    def model(self) -> Model:
        pass

    @property
    @abstractmethod
    def output_schema(self) -> BaseModel:
        pass

    @property
    @abstractmethod
    def filter_schema(self) -> BaseModel:
        pass

    @property
    @abstractmethod
    def acceptable_values(self) -> list:
        pass

    def service_output_handler(self, *args, **kwargs):
        ret = self.__get_from_db()
        return ret

    def convert_to_schema(self, obj):
        ret = self.output_schema(**obj) if obj else None
        return ret

    def convert_to_schema_many_true(self, objs):
        ret = [self.convert_to_schema(i) for i in objs]
        return ret

    def get_from_db(self):
        rslt = self.__query()

        if not self.is_object:
            if self.many:
                rslt = self.convert_to_schema_many_true(self.__query())
            else:
                rslt = self.convert_to_schema(self.__query())

        ret = rslt
        return ret

    def __values(self, *values) -> list:
        if values:
            tmp = set()
            for i in values:
                if i in self.acceptable_values:
                    tmp.add(i)

            values = list(tmp)

        ret = values if values else self.acceptable_values
        return ret

    def __filters(self, **filters) -> dict:
        inst = self.filter_schema(**filters)
        ret = inst.model_dump().extract()
        return ret

    def __many(self, *args, **kwargs) -> bool:
        many = kwargs.get(MANY_KEYWORD)
        if many is True or MANY_KEYWORD in args:
            return True

        return False

    def __first_or_last(self, *args, **kwargs) -> str:
        first = kwargs.get(FIRST_KEYWORD)
        if first is True or FIRST_KEYWORD in args:
            return FIRST_KEYWORD

        return LAST_KEYWORD

    def __is_object(self, **kwargs) -> bool:
        is_object = kwargs.get(OBJECT_TYPE_KEYWORD)
        if is_object is True:
            return True

        return False

    def __query_set(self):
        if self.is_object:
            qs = self.model.objects.filter(**self.filters)
        else:
            qs = self.model.objects.values(*self.values).filter(**self.filters)

        ret = qs
        return ret

    def __query(self):
        if self.many:
            q = self.__query_set().all()

        elif self.first_or_last == LAST_KEYWORD:
            q = self.__query_set().last()

        else:
            q = self.__query_set().first()

        ret = q
        return ret



from .__messages import Messages as MessagesSchema

from .__response import Response as ResponseSchema

from .__data_many_true import DataManyTrueSchema
from .__data_many_false import DataManyFalseSchema



from django.utils.translation import gettext_lazy as _
from django.db import models

from gando.utils.converters.images import small_blur_base64
from gando.utils.uploaders.images import ImageUploadTo


class BlurBase64Field(models.TextField):
    def formfield(self, **kwargs):
        return None

    def pre_save(self, model_instance, add):
        _src = getattr(model_instance, f'{self.PARENT_FIELD_NAME}_src')

        setattr(model_instance, self.attname, small_blur_base64(_src.file.name))
        return super().pre_save(model_instance, add)


class BaseMultiplyField(models.Field):
    def __init__(self, **kwargs):
        self.field_kwargs = kwargs

        super().__init__(**{})

    def sub_field_contribute_to_class(
        self,

        cls,
        field_name,

        sub_field_name,
        sub_filed_class,
        sub_field_default_attr=None,
    ):

        my_kwargs = self.__get_my_kwargs(
            my_name=sub_field_name,
            kwargs=self.field_kwargs,
            default=sub_field_default_attr,
        )

        sub_field = sub_filed_class(
            name=field_name + f'_{sub_field_name}',
            **my_kwargs,
        )

        sub_field.contribute_to_class(cls, sub_field.name)

        setattr(sub_field, 'PARENT_FIELD_NAME', field_name)

    @staticmethod
    def __get_my_kwargs(my_name, kwargs: dict, default: dict = None):
        kwargs_ = {}
        for k, v in kwargs.items():
            if f'{my_name}__' in k:
                kwargs_[k[len(f'{my_name}__'):]] = v

        default_ = default or {}
        for k, v in default_.items():
            if k not in kwargs_:
                kwargs_[k] = v

        return kwargs_


class Image:
    def __init__(self, **kwargs):
        self.alt = kwargs.get('alt')
        self.description = kwargs.get('description')
        self.width = kwargs.get('width')
        self.height = kwargs.get('height')
        self.src = kwargs.get('src')
        self.blur_base64 = kwargs.get('blur_base64')


class ImageProperty:
    def __init__(self, name):
        self.name = name

    def __get__(self, instance, owner):
        if not instance:
            return self

        ret = Image(
            alt=getattr(instance, self.name + '_alt'),
            description=getattr(instance, self.name + '_description'),
            width=getattr(instance, self.name + '_width'),
            height=getattr(instance, self.name + '_height'),
            src=getattr(instance, self.name + '_src'),
            blur_base64=getattr(instance, self.name + '_blur_base64'),
        )
        return ret

    def __set__(self, instance, value: dict):
        src = value.get('src')

        blur_base64 = small_blur_base64(src) if src.name.split('.')[-1] != 'svg' else None

        setattr(
            instance, self.name + '_alt', value.get('alt'))
        setattr(
            instance, self.name + '_description', value.get('description'))
        setattr(
            instance, self.name + '_width', value.get('width'))
        setattr(
            instance, self.name + '_height', value.get('height'))
        setattr(
            instance, self.name + '_src', src)
        setattr(
            instance, self.name + '_blur_base64', blur_base64)


class ImageField(BaseMultiplyField):

    def contribute_to_class(self, cls, name, private_only=False):
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='alt',
            sub_filed_class=models.CharField,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} Alternate Title'),
                'help_text': _('This title will be shown instead if the image is not loaded.'),
                'max_length': 255,
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='description',
            sub_filed_class=models.TextField,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} Description'),
                'help_text': _("If you want a text to be written below the image,"
                               " write it here. Otherwise, leave it blank."),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='width',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} Width'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='height',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} Height'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='src',
            sub_filed_class=models.ImageField,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} SRC'),
                'upload_to': ImageUploadTo(),
                'blank': True,
                'null': True,

                'width_field': f'{name}_width',
                'height_field': f'{name}_height',
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=name,
            sub_field_name='blur_base64',
            sub_filed_class=BlurBase64Field,
            sub_field_default_attr={
                'verbose_name': _(f'{name[0].upper()}{name[1:].lower()} Blur-Base64'),
                'blank': True,
                'null': True,
            }
        )

        setattr(cls, name, ImageProperty(name))


class ImageDeviceField(BaseMultiplyField):

    def contribute_to_class(self, cls, name, private_only=False):
        desktop_name = f'{name}_desktop'

        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='alt',
            sub_filed_class=models.CharField,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} Alternate Title'),
                'help_text': _('This title will be shown instead if the image is not loaded.'),
                'max_length': 255,
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='description',
            sub_filed_class=models.TextField,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} Description'),
                'help_text': _("If you want a text to be written below the image,"
                               " write it here. Otherwise, leave it blank."),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='width',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} Width'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='height',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} Height'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='src',
            sub_filed_class=models.ImageField,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} SRC'),
                'upload_to': ImageUploadTo(),
                'blank': True,
                'null': True,

                'width_field': f'{desktop_name}_width',
                'height_field': f'{desktop_name}_height',
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=desktop_name,
            sub_field_name='desktop_blur_base64',
            sub_filed_class=BlurBase64Field,
            sub_field_default_attr={
                'verbose_name': _(f'{desktop_name[0].upper()}{desktop_name[1:].lower()} Blur-Base64'),
                'blank': True,
                'null': True,
            }
        )

        setattr(cls, desktop_name, ImageProperty(desktop_name))

        mobile_name = f'{name}_mobile'

        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='alt',
            sub_filed_class=models.CharField,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} Alternate Title'),
                'help_text': _('This title will be shown instead if the image is not loaded.'),
                'max_length': 255,
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='description',
            sub_filed_class=models.TextField,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} Description'),
                'help_text': _("If you want a text to be written below the image,"
                               " write it here. Otherwise, leave it blank."),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='width',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} Width'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='height',
            sub_filed_class=models.PositiveIntegerField,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} Height'),
                'blank': True,
                'null': True,
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='src',
            sub_filed_class=models.ImageField,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} SRC'),
                'upload_to': ImageUploadTo(),
                'blank': True,
                'null': True,

                'width_field': f'{mobile_name}_width',
                'height_field': f'{mobile_name}_height',
            }
        )
        self.sub_field_contribute_to_class(
            cls,
            field_name=mobile_name,
            sub_field_name='mobile_blur_base64',
            sub_filed_class=BlurBase64Field,
            sub_field_default_attr={
                'verbose_name': _(f'{mobile_name[0].upper()}{mobile_name[1:].lower()} Blur-Base64'),
                'blank': True,
                'null': True,
            }
        )
        setattr(cls, mobile_name, ImageProperty(mobile_name))



from pydantic import BaseModel

from typing import Any


class AnonymousUser(BaseModel):
    id: int = None
    pk: int = None

    username: str = ''

    user_permissions: Any = None

    is_superuser: bool = False
    is_staff: bool = False
    is_authenticated: bool = False
    is_anonymous: bool = True
    is_active: bool = False

    groups: Any = None


class RequestSchema(BaseModel):
    data: dict = dict()
    query_params: dict = dict()
    method: str
    user: Any = AnonymousUser()
    headers: dict = dict()
    cookies: dict = dict()



from gando.utils.strings.casings import (
    SNAKE_CASE,
    CAMEL_CASE,
    PASCAL_CASE,
    KEBAB_CASE,
    ANY,
)


def casing(value: str, from_case: str = ANY, to_case: str = SNAKE_CASE):
    tmp, start_underscores = get_all_start_underscores(value)
    tmp, end_underscores = get_all_end_underscores(tmp)

    len_tmp = len(tmp)
    if len_tmp == 0:
        return value

    match from_case.lower():
        case 'any':
            match to_case:
                case 'snake':
                    rslt = a2s(tmp)
                case 'camel':
                    rslt = s2c(a2s(tmp))
                case 'pascal':
                    rslt = s2p(a2s(tmp))
                case 'kebab':
                    rslt = s2k(a2s(tmp))
                case _:
                    rslt = tmp
        case 'snake':
            match to_case:
                case 'camel':
                    rslt = s2c(tmp)
                case 'pascal':
                    rslt = s2p(tmp)
                case 'kebab':
                    rslt = s2k(tmp)
                case _:
                    rslt = tmp
        case 'camel':
            match to_case:
                case 'snake':
                    rslt = c2s(tmp)
                case 'pascal':
                    rslt = c2p(tmp)
                case 'kebab':
                    rslt = c2k(tmp)
                case _:
                    rslt = tmp
        case 'pascal':
            match to_case:
                case 'snake':
                    rslt = p2s(tmp)
                case 'camel':
                    rslt = p2c(tmp)
                case 'kebab':
                    rslt = p2k(tmp)
                case _:
                    rslt = tmp
        case 'kebab':
            match to_case:
                case 'snake':
                    rslt = k2s(tmp)
                case 'camel':
                    rslt = k2c(tmp)
                case 'pascal':
                    rslt = k2p(tmp)
                case _:
                    rslt = tmp
        case _:
            rslt = tmp
    ret = start_underscores + rslt + end_underscores
    return ret


def s2c(value: str):
    tmp = ''
    uppercase = False
    for c in value:
        if c == '_':
            uppercase = True
            continue
        tmp += c.upper() if uppercase else c
        uppercase = False
    ret = tmp
    return ret


def s2p(value: str):
    tmp = ''
    uppercase = True
    for c in value:
        if c == '_':
            uppercase = True
            continue
        tmp += c.upper() if uppercase else c
        uppercase = False
    ret = tmp
    return ret


def s2k(value: str):
    ret = value.replace('_', '-')
    return ret


def c2s(value: str):
    tmp = ''
    for c in value:
        tmp += f'_{c.lower()}' if c.isupper() else c
    ret = tmp
    return ret


def c2p(value: str):
    ret = value[0].upper() + value[1:] if len(value) > 1 else ''
    return ret


def c2k(value: str):
    tmp = ''
    for c in value:
        tmp += f'-{c.lower()}' if c.isupper() else c
    ret = tmp
    return ret


def p2s(value: str):
    tmp = value[0].lower()
    for c in value[1:] if len(value) > 1 else '':
        tmp += f'_{c.lower()}' if c.isupper() else c
    ret = tmp
    return ret


def p2c(value: str):
    ret = value[0].lower() + value[1:] if len(value) > 1 else ''
    return ret


def p2k(value: str):
    tmp = value[0].lower()
    for c in value[1:] if len(value) > 1 else '':
        tmp += f'-{c.lower()}' if c.isupper() else c
    ret = tmp
    return ret


def k2s(value: str):
    ret = value.replace('-', '_')
    return ret


def k2c(value: str):
    tmp = ''
    uppercase = False
    for c in value:
        if c == '-':
            uppercase = True
            continue
        tmp += c.upper() if uppercase else c
        uppercase = False
    ret = tmp
    return ret


def k2p(value: str):
    tmp = ''
    uppercase = True
    for c in value:
        if c == '-':
            uppercase = True
            continue
        tmp += c.upper() if uppercase else c
        uppercase = False
    ret = tmp
    return ret


def get_all_start_underscores(value):
    i = 0
    start_underscores = ''
    len_val = len(value)
    while len_val > i and value[i] == '_':
        start_underscores += '_'
        i += 1
    return value[i:] if len_val > i else '', start_underscores


def get_all_end_underscores(value):
    i = -1
    end_underscores = ''
    len_val = len(value)
    while len_val >= abs(i) and value[i] == '_':
        end_underscores += '_'
        i -= 1
    return value[:len_val + i + 1] if len_val >= abs(i) else '', end_underscores


def a2s(value: str):
    value_ = value.replace('-', '_')

    len_val = len(value_)
    if len_val == 1:
        return value_.lower()

    tmp = value_[0].lower()
    for i in value_[1:]:
        tmp += f'_{i.lower()}' if i.isupper() else i

    ret = tmp
    return ret



from simple_history.models import HistoricalRecords
import uuid

from django.utils.translation import gettext_lazy as _
from django.db import models


class AbstractBaseModel(models.Model):
    uid = models.UUIDField(
        verbose_name=_('UID'),
        unique=True,
        default=uuid.uuid4,
        editable=False,
        db_index=True,
    )
    created_dt = models.DateTimeField(
        verbose_name=_('Created Datetime'),
        auto_now_add=True,
        db_index=True,
    )
    updated_dt = models.DateTimeField(
        verbose_name=_('Updated Datetime'),
        auto_now=True,
        db_index=True,
    )
    available = models.IntegerField(
        verbose_name=_('Available'),
        default=1,
        choices=(
            (0, 'No'),
            (1, 'Yes'),
        ),
        db_index=True,
    )

    history = HistoricalRecords(inherit=True)

    class Meta:
        abstract = True
